<?php

namespace Ulmus\Api;

use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\SimpleCache\CacheInterface;
use Ulmus\Adapter\AdapterInterface;
use Ulmus\Api\Request\JsonRequest;
use Ulmus\Api\Response\JsonResponse;
use Ulmus\Api\Response\Response;
use Ulmus\Ulmus;

class ConnectionAdapter extends \Ulmus\ConnectionAdapter
{
    protected AdapterInterface $adapter;

    private array $connection;

    public function __construct(
        public readonly ApiHandlerInterface $apiHandler,
        public string $name = "default",
        protected array $configuration = [],
        public bool $default = false,
        public ? CacheInterface $cacheObject = null
    ) {
        parent::__construct($name, $configuration, $default);
    }

    public function resolveConfiguration(): void
    {
        $connection = $this->configuration['connections'][$this->name] ?? [];

        if (false !== ($adapterName = $connection['adapter'] ?? false)) {
            $this->adapter = $this->instanciateAdapter($adapterName);
        } else {
            $this->adapter = new Adapter\Rest();
        }

        $this->adapter->setup($connection);
    }

    public function getConfiguration(): array
    {
        return $this->configuration['connections'][$this->name];
    }

    /**
     * Connect the adapter
     * @return self
     */
    public function connect(): self
    {
        $this->connection = $this->adapter->connect();

        return $this;
    }

    public function connector(): object
    {
        return isset($this->connection) ? $this->connection : $this->connect()->connection;
    }

    public function adapter(): AdapterInterface
    {
        return $this->adapter;
    }

    protected function instanciateAdapter($name): AdapterInterface
    {
        if (class_exists($name)) {
            return new $name();
        }

        $class = sprintf("\\%s\\Adapter\\%s", __NAMESPACE__, $name);

        return new $class();
    }
}
