<?php

namespace Ulmus\Api;

use Ulmus, Ulmus\Query\QueryFragmentInterface;

class RequestBuilder implements Ulmus\QueryBuilder\QueryBuilderInterface
{
    public Ulmus\QueryBuilder\QueryBuilderInterface $parent;

    public RequestBuilder\Filter $filters;

    public RequestBuilder\UrlParameter $urlParameters;

    protected int $parameterIndex = 0;

    protected array $queryParamStack = [];

    protected ? string $limitQueryField = "limit";

    protected ? string $offsetQueryField = "offset";

    protected ? string $pageQueryField = "page";

    public function values(array $dataset) : self
    {
        $this->addValues(array_values($dataset));

        return $this;
    }

    public function filter(\Stringable|string $field, mixed $value,) : self
    {
        if ( empty($this->filters) && ! $this->getFragment(RequestBuilder\Filter::class) ) {
            $this->filters = new RequestBuilder\Filter();
            $this->push($this->filters);
        }

        $this->filters->add($field, $value);

        return $this;
    }

    public function where(\Stringable|string $field, mixed $value,) : self
    {
        return $this->filter($field, $value);
    }

    public function bindUrl(\Stringable|string $field, mixed $value,) : self
    {
        if ( empty($this->urlParameters) && ! $this->getFragment(RequestBuilder\UrlParameter::class) ) {
            $this->urlParameters = new RequestBuilder\UrlParameter();
            $this->push($this->urlParameters);
        }

        $this->urlParameters->add($field, $value);

        return $this;
    }

    public function limit(int $value) : self
    {
        return is_null($this->limitQueryField) ? $this : $this->filter($this->limitQueryField, $value);
    }

    public function offset(int $value) : self
    {
        return is_null($this->offsetQueryField) ? $this : $this->filter($this->offsetQueryField, $value);
    }

    public function page(int $value) : self
    {
        return is_null($this->pageQueryField) ? $this : $this->filter($this->pageQueryField, $value);
    }


    public function orderBy(string $field, ? string $direction = null) : self
    {
        return $this->where($field, $direction);
    }

    public function push(QueryFragmentInterface $queryFragment) : self
    {
        $this->queryParamStack[] = $queryFragment;

        return $this;
    }

    public function pull(QueryFragmentInterface $queryFragment) : self
    {
        return array_shift($this->queryParamStack);
    }

    public function render(bool $skipToken = false) : array
    {
        $stack = [];

        foreach($this->queryParamStack as $fragment) {
            $stack = array_merge($stack, (array) $fragment->render($skipToken));
        }

        return $stack;
    }

    public function reset() : void
    {
        unset($this->filters);
    }

    public function getFragment(string $class, int $index = 0) : ? QueryFragmentInterface
    {
        foreach($this->queryParamStack as $item) {
            if ( get_class($item) === $class ) {
                if ( $index-- === 0 ) {
                    return $item;
                }
            }
        }

        return null;
    }

    public function removeFragment(QueryFragmentInterface|array|\Stringable|string $fragment) : void
    {
        foreach($this->queryParamStack as $key => $item) {
            if ( $item === $fragment ) {
                unset($this->queryParamStack[$key]);
            }
        }
    }

    public function __toString() : string
    {
        return $this->render();
    }

    public function hashSerializedQuery(): string
    {
        return "UNSUPPORTED";
    }
}
